/*********************************************************************************************************************
 Copyright Amazon.com, Inc. or its affiliates. All Rights Reserved.

 Licensed under the Apache License, Version 2.0 (the "License").
 You may not use this file except in compliance with the License.
 You may obtain a copy of the License at

 http://www.apache.org/licenses/LICENSE-2.0

 Unless required by applicable law or agreed to in writing, software
 distributed under the License is distributed on an "AS IS" BASIS,
 WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 See the License for the specific language governing permissions and
 limitations under the License.
 ******************************************************************************************************************** */
import { Project } from "projen";
import { TypeScriptProject, TypeScriptProjectOptions } from "projen/lib/typescript";
/**
 * Configuration for nx targetDependencies.
 */
export declare type TargetDependencies = {
    [target: string]: TargetDependency[];
};
/**
 * Implicit Dependencies map.
 */
export declare type ImplicitDependencies = {
    [pkg: string]: string[];
};
/**
 * Supported enums for a TargetDependency.
 */
export declare enum TargetDependencyProject {
    /**
     * Only rely on the package where the target is called.
     *
     * This is usually done for test like targets where you only want to run unit
     * tests on the target packages without testing all dependent packages.
     */
    SELF = "self",
    /**
     * Target relies on executing the target against all dependencies first.
     *
     * This is usually done for build like targets where you want to build all
     * dependant projects first.
     */
    DEPENDENCIES = "dependencies"
}
/**
 * Represents an NX Target Dependency.
 */
export interface TargetDependency {
    /**
     * Projen target i.e: build, test, etc
     */
    readonly target: string;
    /**
     * Target dependencies.
     */
    readonly projects: TargetDependencyProject;
}
/**
 * NX configurations.
 *
 * @link https://nx.dev/configuration/packagejson
 */
export interface NXConfig {
    /**
     * Configuration for Implicit Dependnecies.
     *
     * @link https://nx.dev/configuration/packagejson#implicitdependencies
     */
    readonly implicitDependencies?: ImplicitDependencies;
    /**
     * Configuration for TargetDependencies.
     *
     * @link https://nx.dev/configuration/packagejson#target-dependencies
     */
    readonly targetDependencies?: TargetDependencies;
    /**
     * List of patterns to include in the .nxignore file.
     *
     * @link https://nx.dev/configuration/packagejson#nxignore
     */
    readonly nxIgnore?: string[];
}
/**
 * Workspace configurations.
 *
 * @link https://classic.yarnpkg.com/lang/en/docs/workspaces/
 */
export interface WorkspaceConfig {
    /**
     * List of package globs to exclude from hoisting in the workspace.
     *
     * @link https://classic.yarnpkg.com/blog/2018/02/15/nohoist/
     */
    readonly noHoist?: string[];
}
/**
 * Configuration options for the NxMonorepoProject.
 */
export interface NxMonorepoProjectOptions extends TypeScriptProjectOptions {
    /**
     * Configuration for NX.
     */
    readonly nxConfig?: NXConfig;
    /**
     * Configuration for workspace.
     */
    readonly workspaceConfig?: WorkspaceConfig;
}
/**
 * This project type will bootstrap a NX based monorepo with support for polygot
 * builds, build caching, dependency graph visualization and much more.
 *
 * @pjid nx-monorepo
 */
export declare class NxMonorepoProject extends TypeScriptProject {
    private readonly implicitDependencies;
    private readonly nxConfig?;
    private readonly workspaceConfig?;
    constructor(options: NxMonorepoProjectOptions);
    /**
     * Create an implicit dependency between two Project's. This is typically
     * used in polygot repos where a Typescript project wants a build dependency
     * on a Python project as an example.
     *
     * @param dependent project you want to have the dependency.
     * @param dependee project you wish to depend on.
     */
    addImplicitDependency(dependent: Project, dependee: Project): void;
    protected get subProjects(): Project[];
    /**
     * @inheritDoc
     */
    synth(): void;
    /**
     * Ensures subprojects don't have a default task and that all packages use the same package manager.
     */
    private validateSubProjects;
    /**
     * For non-node projects, a package.json is required in order to be discovered by NX.
     */
    private synthesizeNonNodePackageJson;
    /**
     * Add a submodule entry to the appropriate workspace file.
     */
    private updateWorkspace;
}
//# sourceMappingURL=nx-monorepo.d.ts.map